
    comment * This routine handles much of the interface to the Dongle reading
    in the Parameter Basket and the Data Basket.  As written, it is designed to
    be included inline rather than linked. The assume CS: gives us access to
    all of the variables (the assume is in amdflash.asm). All symbols are
    "global" as we use the include mechanism to make one compact output file.


    process_basket_header:  Fetch header from basket and display information
    based on it, also save basket size to variable BasketSize.


    process_RLE_header:  Fetch RLE header from basket, if basket type suggests
    RLE image.  If this is a compressed basket, change variable "basketsize"
    ResetCRC is called prior to this routine so INT 17H can be used

    Create_Memory_Window:  Create a memory window for access to the flash using
    global equates CS_SIZE, PRGBASE


    Set_Window_Mode:  Set mode for specified chipselect using
    global equates WINDOWMODE, MEMC

    Subroutine Set_Flash_Offset: Set Memory Window PAGE Register by reading
    BASE from the ZF registers, and using global variable FlashBase. The
    formula is PAGE = FlashBase + 0x1000000 - BASE.



    revision history

    CRC 12-26-00        updated comments, rearranged code


    Header Structure for ZTAG Dongle Data Record

    ; db 07Fh,0F0h,055h        ; command start ID
    ; db 01h                   ; command code
    ; db 019h,099h,010h,021h   ; BCD date = 1999.10.21
    ; db 023h,059h             ; BCD time = 23:59
    ; db 6                     ; description string length
    : db 'Sample'              ; description string (23 bytes max!)
    ; db 001h,030h             ; BCD version = 1.30
    ; dd 00000100              ; body length
    ; db body_length dup (?)   ; body
    ; dw E2FC                  ; 16-bit CRC of body

    modified 12-26-00 CRC

endcomment *


SMessage1       db      'Source: ',0    ; subs message 1
SMessage2       db      ' V',0
SMessage3       db      ' (0x',0
SMessage4       db      'Basket is a RLE encoded image.',CR,LF,0
SMessage5       db      'RLE basket does not contain valid RLE compression header!',CR,LF,0
SMessage6       db      'RLE Image header reports different size from basket!',CR,LF,0
SMessage7       db      'No Payload. Exiting.',CR,LF,0
SMessage8       db      ' bytes)',CR,LF,0

failureOffset   dw      ?       ; some calls pass in <error offset>

;   --------------------------------
;   Subroutine process_basket_header
;   --------------------------------

; Fetch header from basket and display information based on it, also
; save basket size to variable BasketSize.


process_basket_header proc
                mov     failureOffset, BX

                ; Fetch header now - get < command start ID> and <command code>

                mov     cx, 4
GetTheHeader:
                shl     eax, 8          ; hopefully create 7FF055FF or 7FF055F0
                call    ZTRead          ; by reading 4 bytes
                loop    GetTheHeader

                mov     BasketType, al  ; save basket type as FF or F0 we hope

                cmp     eax, 7FF055FFh  ; is that a basket?
                jz      foundbasket
                cmp     eax, 7FF055F0h  ; RLE basket?
                jz      foundbasket

                LEA     DI, Smessage7   ; 'No Payload. Exiting.'
                CALL    Print
                POP     AX              ; restore stack for no good reason
                jmp     failureOffset   ; jump out to failure address
foundbasket:

    ; Discard date and time which is 6 bytes (example below)

    ; db 019h,099h,010h,021h   ; BCD date = 1999.10.21
    ; db 023h,059h             ; BCD time = 23:59

                mov     cx, 6
DiscardTheTime:
                call    ZTRead
                loop    DiscardTheTime

    ; Output basket name (example below)
    ; db  6                    ; description string length
    ; db 'Sample'              ; description string (23 bytes max!)

                LEA     DI, SMessage1   ; 'Source: '
                call    print

                call    ZTRead          ; BUR API routine
                xor     cx, cx          ; cx = 0
                mov     cl, al          ; string length is 1 byte
ShowDesc:
                call    ZTRead          ; BUR API routine
                call    SerSend2        ; BUR API routine
                loop    ShowDesc        ; print CX characters

    ; Show version (example below)
    ; db 001h,030h             ; BCD version = 1.30

                LEA     DI, Smessage2   ; ' V'
                call    print

                call    ZTRead          ; BUR API routine
                call    SerOut8         ; BUR API routine display 8-bit hex number
                mov     al, '.'         ;
                call    SerSend2        ; BUR API routine char to active serial console
                call    ZTRead          ; BUR API routine
                call    SerOut8         ; BUR API routine

    ; Show size
    ; dd 00000100              ; body length
    ; db body_length dup (?)   ; body

                LEA     DI, Smessage3   ; ' (0x'
                call    print

                mov     cx, 4
ReadSize:
                shl     eax, 8
                call    ZTRead
                loop    ReadSize

                bswap   eax             ; change endian to get bytes in right order
                mov     basketsize, eax ; variable in flashpgm.asm
                call    SerOut32        ; BUR API Routine display 32-bit hex number

                LEA     DI, smessage8   ; ' bytes)'
                CALL    Print
                RET
process_basket_header endp


comment * -------------------------------------------------------------- *
; subroutine process_RLE_header
;
; Fetch RLE header from basket, if basket type suggests RLE image.
; If this is a compressed basket, change variable "basketsize"
; ResetCRC is called prior to this routine so INT 17H can be used
;

FailOffset      DW      0               ; address of failure label goes here

process_RLE_header proc
                mov     FailOffset, AX  ; fail jump-to offset passed in

                ; Final check for RLE image

                cmp     BasketType, 0F0h
                jnz     NotCompressed

;Compressed:
                LEA     DI, Smessage4   ; 'Basket is a RLE encoded image.'
                CALL    print

                call    ZTPrepareRead   ; in BURAPI.ASM

                ; See if basket itself thinks it is RLE image?

                mov     cx, 4           ; initial value of eax not important
getfourbytes:
                shl     eax, 8          ; shift in previous of 4 reads...
                call    ZTRead          ; get one byte
                int     17h             ; provded by BUR in RAM - Fast CRC Calculation
                loop    getfourbytes    ;

                cmp     eax, '$RLE'     ; this is a special RLE compression header
                jz      RLEHeaderOK     ;  not documented yet...

                LEA     DI, Smessage5   ;  'RLE basket does not contain valid RLE compression header!'
                CALL    Print
                POP     EAX             ; restore stack for no good reason
                jmp     FailOffset      ; indirect jump
RLEHeaderOK:
                ; RLE header was OK. Fetch the rest of it

                mov     di, offset RLEHeader    ; "structure" in FlashPGM.asm
                mov     cx, 11h
fetch11hloop:
                call    ZTRead                  ; in burapi.asm
                int     17h                     ; provied by BUR in RAM - Fast CRC Calculation
                mov     byte ptr cs:[di], al
                inc     di
                loop    fetch11hloop

                ; Check, if RLL-compressed image length matches the one reported by
                ; ZTAG record header

                mov     eax, BasketSize
                sub     eax, 15h                ; length of '$RLE'+11h bytes of data
                cmp     eax, RLE_CompLen        ; variable in FlashPgm.ASM part of "structure" RLEHeader
                jz      RLE_LEN_OK

                LEA     DI, Smessage6   ; 'RLE Image header reports different size from basket!'
                CALL    print
                POP     EAX             ; restore stack for no good reason
                jmp     FailOffset      ; indirect jump
RLE_LEN_OK:
                ; lengths do match, so replace BasketSize with uncompressed length.
                ; This way the writing loop calls ZTRead_RLE correct number of times

                mov     eax, RLE_UnCompLen
                mov     BasketSize, eax

NotCompressed:  ret

process_RLE_header endp



;   -------------------------------
;   Subroutine Create_Memory_Window
;   -------------------------------
;
; Create memory window (PRGBASE). Size will be 64K (hardwired)
;
Create_Memory_Window proc

                push    dx
                push    eax

                ; Create Memory window

                mov     dx, ZFINDEX
                mov     al, CS_BASE     ; base register for chipselect
                out     dx, al
                mov     dx, ZFRW1632
                mov     eax, PRGBASE    ; set window base
                out     dx, eax

                ; Set Window size

                mov     dx, ZFINDEX
                mov     al, CS_SIZE     ; size address for chipselect
                out     dx, al
                mov     dx, ZFRW1632
                mov     eax, 64*1024    ; 64K window
                dec     eax
                out     dx, eax

                Call    Set_Flash_Offset        ; uses MEMCS, Flashbase
                Call    Set_Window_Mode

                pop     eax
                pop     dx
                ret

Create_Memory_Window endp

;   --------------------------
;   subroutine Set_Window_Mode
;   --------------------------
;
; Set mode for specified chipselect (WINDOWMODE, MEMCS)
;
Set_Window_Mode proc
                push    ax
                push    dx

                ; Set chip to correct width and RW mode

                mov     dx, ZFINDEX
                mov     al, MEMCONTROLL
                out     dx, al                  ; point to Memory Control Low
                mov     dx, ZFRW8               ; read byte in
                in      al, dx

                mov     ah, 11101110b           ; mask out the R/W and Width Bits
                rol     ah, MEMCS
                and     al, ah                  ; write applicable bits to 0 first

                mov     ah, WINDOWMODE          ; 8-bit read/write for AMD chip in IDS
                rol     ah, MEMCS               ; move mode bits to correct chipselect position
                or      al, ah

                out     dx, al                  ; write out applicable bits

                pop     dx
                pop     ax
                ret
Set_Window_Mode endp



;   --------------------------
;   Subroutine Set_Flash_Offset
;   --------------------------
;
; Set Memory Window PAGE Register
;
; PAGE = FlashBase + 0x1000000 - BASE

Set_Flash_Offset proc
                push    eax
                push    ebx
                push    dx

                ; calculate page address so that the base address in flash would go where it was requested

                mov     dx, ZFINDEX
                mov     al, CS_BASE
                out     dx, al
                mov     dx, ZFRW1632
                in      eax, dx                 ; determine base address
                mov     ebx, eax                ; and save it in ebx

                mov     dx, ZFINDEX
                mov     al, CS_PAGE             ; page address for chipselect
                out     dx, al
                mov     dx, ZFRW1632
                mov     eax, FlashBase
                add     eax, 01000000h          ; set page to cause correct flash address
                sub     eax, ebx                ; substract base address
                out     dx, eax                 ; PAGE = FlashBase + 0x1000000 - BASE

                pop     dx
                pop     ebx
                pop     eax
                ret
Set_Flash_Offset endp

